@@ -22,3 +22,17 @@ class ConflictSignatureError(Exception):
 class InvalidThreadWaitError(Exception):
     def __init__(self):
         super().__init__("the thread wait is invalid, caused by not get lock")
+
+class SpiritContainerServiceException(Exception):
+    def __init__(self):
+        super().__init__("the spirit container service is start, please do not repeat start")
+
+
+class BusinessException(Exception):
+    def __init__(self, msg):
+        super(BusinessException, self).__init__(msg)
+
+
+class SystemException(Exception):
+    def __init__(self, msg):
+        super(SystemException, self).__init__(msg)
